import React__default, { useCallback, useMemo, useEffect, useState, createElement } from 'react';
import styled from 'styled-components';
import { useTable, useFilters, useGlobalFilter } from 'react-table';
import { lighten } from 'polished';
import { get, isFunction } from 'lodash-es';
import { useBottomScrollListener } from 'react-bottom-scroll-listener';
import DatePickerDefault from 'react-datepicker';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __makeTemplateObject(cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
}

/*
columnsWidth examples
[25, 50, 25]
[25, 'auto', 25]
'auto' considered as '1fr'
 */
var isColumnAString = function (columns) { return typeof columns[0] === 'string'; };
var RowWrapper = function (_a) {
    var row = _a.row, editable = _a.editable, onRowUpdate = _a.onRowUpdate, onRowDelete = _a.onRowDelete, highlight = _a.highlight, rowStyles = _a.rowStyles, onRowClick = _a.onRowClick, isSelected = _a.isSelected;
    var _b = useState(false), editing = _b[0], setEditing = _b[1];
    var _c = useState(row.original), values = _c[0], setValues = _c[1];
    useEffect(function () {
        setValues(row.original);
    }, [row.original]);
    var handleChange = function (key) { return function (value) {
        setValues(function (prevState) {
            var _a;
            return (__assign(__assign({}, prevState), (_a = {}, _a[key] = value, _a)));
        });
    }; };
    var handleClick = function (data) { return function (event) {
        event.stopPropagation();
        isFunction(onRowClick) && onRowClick(data.original);
    }; };
    var handleDoubleClick = function () {
        if (editable) {
            setEditing(true);
        }
    };
    var handleEnterPress = function (event) {
        if (event.key === 'Enter') {
            onRowUpdate(values);
            setEditing(function (prevState) { return !prevState; });
        }
    };
    return (React__default.createElement(StyledTr, __assign({}, row.getRowProps(), { onClick: handleClick(row), styles: rowStyles, onDoubleClick: handleDoubleClick, isSelected: isFunction(isSelected) && isSelected(row.original) }), row.cells.map(function (cell, index, cells) {
        return editable ? (React__default.createElement(StyledTd, __assign({}, cell.getCellProps(), { key: cell.column.id }),
            cell.render('Cell', {
                edit: editing,
                value: values[cell.column.id],
                onChange: handleChange(cell.column.id),
                onEnterPress: handleEnterPress,
                highlight: highlight,
            }),
            index + 1 === cells.length && (React__default.createElement(ActionsWrapper, { active: editing })))) : (React__default.createElement(StyledTd, __assign({}, cell.getCellProps()), cell.render('Cell')));
    })));
};
var Table = function (_a) {
    var columns = _a.columns, data = _a.data, onRowClick = _a.onRowClick, editable = _a.editable, onRowUpdate = _a.onRowUpdate, onRowDelete = _a.onRowDelete, searchQuery = _a.searchQuery, rowStyles = _a.rowStyles, tableStyles = _a.tableStyles, rowId = _a.rowId, onBottom = _a.onBottom, isSelected = _a.isSelected, columnsWidth = _a.columnsWidth;
    // const { t } = useTranslation(); TODO integrate locale
    var t = useCallback(function (s) { return s; }, []);
    var handleBottomScroll = useCallback(function () {
        if (onBottom) {
            onBottom();
        }
    }, [onBottom]);
    var scrollRef = useBottomScrollListener(handleBottomScroll, 0, 200); // cb, offset and debounce
    var processedColumns = useMemo(function () {
        if (isColumnAString(columns)) {
            return columns.map(function (column) { return ({
                Header: String(t(column)),
                accessor: column,
            }); });
        }
        else {
            return columns;
        }
    }, [columns, t]);
    var _b = useTable({
        columns: processedColumns,
        data: data,
    }, useFilters, useGlobalFilter), getTableProps = _b.getTableProps, getTableBodyProps = _b.getTableBodyProps, headerGroups = _b.headerGroups, rows = _b.rows, prepareRow = _b.prepareRow, 
    // @ts-ignore
    // Outdated @types library
    setGlobalFilter = _b.setGlobalFilter;
    useEffect(function () {
        setGlobalFilter(searchQuery);
    }, [searchQuery, setGlobalFilter]);
    return (React__default.createElement(Wrapper, { ref: scrollRef },
        React__default.createElement(StyledTable, __assign({}, getTableProps(), { styles: tableStyles, fixed: !!(columnsWidth === null || columnsWidth === void 0 ? void 0 : columnsWidth.length) }),
            React__default.createElement(StyledThead, null, headerGroups.map(function (headerGroup) { return (React__default.createElement("tr", __assign({}, headerGroup.getHeaderGroupProps()), headerGroup.headers.map(function (column, index) { return (React__default.createElement(StyledTh, __assign({}, column.getHeaderProps(), { size: get(columnsWidth, "[" + index + "]") }), column.render('Header'))); }))); })),
            React__default.createElement("tbody", __assign({}, getTableBodyProps()), rows.map(function (row) {
                prepareRow(row);
                return (React__default.createElement(RowWrapper, { key: rowId ? row.values[rowId] : row.id, row: row, editable: !!editable, onRowUpdate: onRowUpdate, onRowDelete: onRowDelete, highlight: searchQuery, rowStyles: rowStyles, onRowClick: onRowClick, isSelected: isSelected }));
            })))));
};
var Wrapper = styled.div(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  width: 100%;\n  max-height: 100%;\n  overflow: auto;\n"], ["\n  width: 100%;\n  max-height: 100%;\n  overflow: auto;\n"])));
var StyledTable = styled.table(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n  border: 1px solid #9cafb7;\n  border-top: none;\n  width: 100%;\n  border-collapse: collapse;\n\n  font-size: 14px;\n  ", "\n  ", "\n"], ["\n  border: 1px solid #9cafb7;\n  border-top: none;\n  width: 100%;\n  border-collapse: collapse;\n\n  font-size: 14px;\n  ", "\n  ", "\n"])), function (props) { return (props.fixed ? "table-layout: fixed;" : ""); }, function (props) { return props.styles || ''; });
var StyledThead = styled.thead(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n  td {\n    box-shadow: inset 1px -1px #9cafb7;\n  }\n\n  th {\n    position: sticky;\n    top: 0px;\n    z-index: 1;\n    box-shadow: inset 1px 1px #9cafb7, 0 1px #9cafb7;\n  }\n"], ["\n  td {\n    box-shadow: inset 1px -1px #9cafb7;\n  }\n\n  th {\n    position: sticky;\n    top: 0px;\n    z-index: 1;\n    box-shadow: inset 1px 1px #9cafb7, 0 1px #9cafb7;\n  }\n"])));
var StyledTh = styled.th(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n  padding: 3px 7px;\n  font-weight: 500;\n  color: #666666;\n  height: 32px;\n  background-color: #e6e6e6;\n  white-space: nowrap;\n  ", "\n"], ["\n  padding: 3px 7px;\n  font-weight: 500;\n  color: #666666;\n  height: 32px;\n  background-color: #e6e6e6;\n  white-space: nowrap;\n  ",
    "\n"])), function (props) {
    return props.size
        ? "width: " + (props.size === 'auto' ? "auto" : props.size + "%") + ";"
        : "";
});
var StyledTr = styled.tr(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n  background-color: white;\n\n  transition: background-color 0.2s ease;\n  &:hover {\n    background-color: #e8eef2;\n  }\n  ", "\n  ", "\n"], ["\n  background-color: white;\n\n  transition: background-color 0.2s ease;\n  &:hover {\n    background-color: #e8eef2;\n  }\n  ", "\n  ", "\n"])), function (props) { return props.isSelected && 'background-color: rgba(196, 196, 196, 0.2);'; }, function (props) { return props.styles || ''; });
var StyledTd = styled.td(templateObject_6 || (templateObject_6 = __makeTemplateObject(["\n  padding: 3px 7px;\n  height: 32px;\n  border: 1px solid ", ";\n  position: relative;\n  ", "\n"], ["\n  padding: 3px 7px;\n  height: 32px;\n  border: 1px solid ", ";\n  position: relative;\n  ", "\n"])), lighten(0.1, '#9cafb7'), function (props) { return props.styles || ''; });
var ActionsWrapper = styled.div(templateObject_7 || (templateObject_7 = __makeTemplateObject(["\n  position: absolute;\n  right: 10px;\n  top: 50%;\n  transform: translateY(-50%);\n  display: grid;\n  grid-template-columns: repeat(2, auto);\n  grid-column-gap: 10px;\n  justify-content: center;\n\n  svg {\n    opacity: 0.5;\n    cursor: pointer;\n    transition: opacity 0.2s ease;\n    &:hover {\n      opacity: 0.7;\n    }\n  }\n"], ["\n  position: absolute;\n  right: 10px;\n  top: 50%;\n  transform: translateY(-50%);\n  display: grid;\n  grid-template-columns: repeat(2, auto);\n  grid-column-gap: 10px;\n  justify-content: center;\n\n  svg {\n    opacity: 0.5;\n    cursor: pointer;\n    transition: opacity 0.2s ease;\n    &:hover {\n      opacity: 0.7;\n    }\n  }\n"])));
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5, templateObject_6, templateObject_7;

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var _ref = /*#__PURE__*/createElement("path", {
  fillRule: "evenodd",
  clipRule: "evenodd",
  d: "M15.71 2.632c.39.39.39 1.02 0 1.41l-1.83 1.83-3.75-3.75 1.83-1.83a.996.996 0 011.41 0l2.34 2.34zM0 15.504v-3.04c0-.14.05-.26.15-.36l8.91-8.91 3.75 3.75-8.92 8.91a.47.47 0 01-.35.15H.5c-.28 0-.5-.22-.5-.5z",
  fill: "#333"
});

function SvgEdit(props) {
  return /*#__PURE__*/createElement("svg", _extends({
    width: 17,
    height: 17,
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, props), _ref);
}

// Generated with util/create-component.js
var Test2 = function (_a) {
    var foo = _a.foo;
    return (React__default.createElement("div", { "data-testid": "Test2", className: "foo-bar" },
        foo,
        React__default.createElement(SvgEdit, null)));
};

var DatePicker = function (_a) {
    var value = _a.value, onChange = _a.onChange, customInput = _a.customInput, onBlur = _a.onBlur, showTimeSelect = _a.showTimeSelect, showTimeSelectOnly = _a.showTimeSelectOnly, timeIntervals = _a.timeIntervals, dateFormat = _a.dateFormat, timeCaption = _a.timeCaption;
    return (React__default.createElement(DatePickerDefault, { selected: value, onChange: onChange, customInput: customInput, onBlur: onBlur, showTimeSelect: showTimeSelect, showTimeSelectOnly: showTimeSelectOnly, timeIntervals: timeIntervals, dateFormat: dateFormat, timeCaption: timeCaption }));
};

export { DatePicker, Table as TableComponent, Test2 };
//# sourceMappingURL=index.esm.js.map
